"""
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""
import logging
import os
import shutil
import sys
import tarfile
import urllib.error
import urllib.parse
import urllib.request
from abc import ABCMeta, abstractmethod
from tempfile import NamedTemporaryFile
from typing import Optional

from pytmpdir.Directory import Directory
from twisted.internet import reactor, defer
from twisted.internet.defer import inlineCallbacks
from txhttputil.downloader.HttpFileDownloader import HttpFileDownloader
from txhttputil.site.SpooledNamedTemporaryFile import SpooledNamedTemporaryFile

from peek_platform import PeekPlatformConfig
from peek_platform.file_config.PeekFileConfigPlatformMixin import (
    PeekFileConfigPlatformMixin,
)
from peek_platform.util.PtyUtil import spawnPty, logSpawnException
from vortex.DeferUtil import deferToThreadWrapWithLogger

logger = logging.getLogger(__name__)

PLUGIN_PACKAGE_JSON = "plugin_package.json"


class PluginSwInstallManagerABC(metaclass=ABCMeta):
    """Plugin Software Install Manager ABC

    This class handles the downloading and installing of platform plugin updates.

    """

    def __init__(self):
        pass

    @classmethod
    def makePipArgs(cls, fileName: str) -> [str]:
        """Make PIP Args

        This method creates the install arg list for pip, it's used for
        testing and installing

        :param fileName: The full fileName of the package to install
        :return: The list of arguments to pass to pip
        """

        # Create and return the pip args
        return [
            "install",  # Install the packages
            "--force-reinstall",  # Reinstall if they already exist
            "--no-cache-dir",  # Don't use the local pip cache
            "--no-index",  # Work offline, don't use pypi
            fileName,
        ]

    @classmethod
    def getPackageInfo(cls, directory: Directory) -> (str, str):
        """Get Package Info

        Find the PKG-INFO file in the directory and returns the package name and version

        :param directory: The directory where the PyPI style package is extracted to.
        :return: A tuple of the form (pkgName, pkgVersion)
        """

        # CHECK 1
        files = [f for f in directory.files if f.name == "PKG-INFO"]

        if not files:
            raise Exception("Unable to find PKG-INFO")

        pkgInfoFile = files[0]

        # CHECK 2
        pkgName = None
        pkgVersion = None
        with pkgInfoFile.open() as f:
            for line in f:
                if line.startswith("Name: "):
                    pkgName = line.split(":")[1].strip()

                if line.startswith("Version: "):
                    pkgVersion = line.split(":")[1].strip()

        if not pkgName:
            raise Exception("Unable to determine package name")

        if not pkgVersion:
            raise Exception("Unable to determine package version")

        return pkgName, pkgVersion

    @inlineCallbacks
    def update(self, pluginName: str, targetVersion: str) -> Optional[str]:
        """Update

        This method is called when this service detects that a newer version of a plugin
        is availible.

        :param pluginName: The name of the plugin to update
        :param targetVersion: The version to update to.
        :return: The version updated to.
        """

        logger.info("Updating %s to %s", pluginName, targetVersion)

        from peek_platform import PeekPlatformConfig

        url = (
            "http://%(ip)s:%(port)s/peek_logic_service.sw_install.plugin.download?"
        ) % {
            "ip": PeekPlatformConfig.config.peekServerHost,
            "port": PeekPlatformConfig.config.peekServerPort,
        }

        args = {"name": pluginName}
        if targetVersion:
            args["version"] = str(targetVersion)

        url += urllib.parse.urlencode(args)

        try:

            spooledNamedTempFile = yield HttpFileDownloader(url).run()
            assert isinstance(spooledNamedTempFile, SpooledNamedTemporaryFile)

            if os.path.getsize(spooledNamedTempFile.name) == 0:
                logger.warning(
                    "Peek server doesn't have any updates for plugin %s, version %s",
                    pluginName,
                    targetVersion,
                )
                return

            # We need the file to end in .tar.gz
            # PIP doesn't like it otherwise
            namedTempTarGzFile = NamedTemporaryFile(
                dir=PeekPlatformConfig.config.tmpPath, suffix=".tar.gz"
            )
            shutil.copy(spooledNamedTempFile.name, namedTempTarGzFile.name)
            del spooledNamedTempFile

            # We need the file to end in .tar.gz
            # PIP doesn't like it otherwise
            if not namedTempTarGzFile.name.endswith(".tar.gz"):
                shutil.move(
                    namedTempTarGzFile.name, namedTempTarGzFile.name + ".tar.gz"
                )
                namedTempTarGzFile.name += ".tar.gz"

            yield self.installAndReload(
                pluginName, targetVersion, namedTempTarGzFile.name
            )

        except Exception as e:
            logger.exception(e)
            raise

        defer.returnValue(targetVersion)

    @deferToThreadWrapWithLogger(logger)
    def installAndReload(
        self, pluginName: str, targetVersion: str, fullTarPath: str
    ) -> None:

        assert isinstance(PeekPlatformConfig.config, PeekFileConfigPlatformMixin)

        if not tarfile.is_tarfile(fullTarPath):
            raise Exception("Plugin update %s download is not a tar file" % pluginName)

        directory = Directory()
        tarfile.open(fullTarPath).extractall(directory.path)
        directory.scan()

        # CHECK 1
        pgkName, pkgVersion = self.getPackageInfo(directory)

        if pkgVersion != targetVersion:
            raise Exception(
                "Plugin %s trget version is %s actual version is %s"
                % (pluginName, targetVersion, pkgVersion)
            )

        self._pipInstall(fullTarPath)

        PeekPlatformConfig.config.setPluginVersion(pluginName, targetVersion)

        ####
        # FIXME : This will always enabled the Plugin and overwrite config changes
        PeekPlatformConfig.config.pluginsEnabled = list(
            set(PeekPlatformConfig.config.pluginsEnabled + [pluginName])
        )

        # RELOAD PLUGIN
        reactor.callLater(
            0, self.notifyOfPluginVersionUpdate, pluginName, targetVersion
        )

    def _pipInstall(self, fileName: str) -> None:
        """Pip Install Plugin

        Runs the PIP install for the packages provided in the directory

        :param fileName: The full path and filename of the package to install
        :return: None

        """

        pipExec = os.path.join(os.path.dirname(sys.executable), "pip")

        pipArgs = [sys.executable, pipExec] + self.makePipArgs(fileName)

        # # The platform update is tested for dependencies when it's first uploaded
        # # PIP has a bug, when you have updated packages for several dependent files
        # #   and try to install them all at once, some of the packages don't update.
        # pipArgs += ['--no-deps']

        pipArgs = " ".join(pipArgs)

        try:
            spawnPty(pipArgs)
            logger.info("Peek plugin update complete.")

        except Exception as e:
            logSpawnException(e)

            # Update the detail of the exception and raise it
            e.message = "Failed to install new plugin package."
            raise

    @abstractmethod
    def notifyOfPluginVersionUpdate(self, pluginName: str, targetVersion: str) -> None:
        """Notify of Plugin Version Update

        This method is called when a package update has comleted, it notifies the
        service (implementer).

        :param pluginName: The name of the plugin being updated
        :param targetVersion: The version being updated to
        """
        pass



import logging
from typing import List

import os
import shutil
from collections import namedtuple

from peek_platform.build_common.BuilderABC import BuilderABC
from peek_platform.build_frontend.FrontendFileSync import FrontendFileSync
from peek_plugin_base.PluginPackageFileConfig import PluginPackageFileConfig

logger = logging.getLogger(__name__)

PluginDocDetail = namedtuple("PluginDocDetail",
    ["pluginRootDir",
     "pluginName",
     "pluginTitle",
     "docDir",
     "docRst",
     "hasApi"])

_mainTocTemplate = '''

.. toctree::
    :maxdepth: 3
    :caption: Contents:


'''

_tocTemplate = '''

.. toctree::
    :maxdepth: 2
    :caption: Contents:

'''

_confPluginsTemplate = '''

def load(createApiDocsFunc):
    """ Load

        Create APIs with the AutoAPI hack above
    """

'''

_confPluginsTemplatePart = '''
    import %s
    createApiDocsFunc(%s.__file__)

'''


class DocBuilderABC(BuilderABC):
    """ Peek Documentation Builder Mixin

    This class is used for the client and server.

    This class contains the logic for:
        * Linking in the sphinx documentation into the peek-field-doc, peek-admin-doc
            or peek-dev projects
        * Compiling the documentation

    """

    def __init__(self, docProjectDir: str,
                 platformService: str,
                 jsonCfg,
                 loadedPlugins: List):
        assert platformService in (
            "peek-field-doc", "peek-office-doc", "peek-admin-doc"), (
            "Unexpected service %s" % platformService)

        self._platformService = platformService
        self._docProjectDir = docProjectDir
        self._jsonCfg = jsonCfg
        self._loadedPlugins = loadedPlugins

        self._configKey = platformService.replace("peek-", "")

        if not os.path.isdir(docProjectDir):
            raise Exception("%s doesn't exist" % docProjectDir)

        self.fileSync = FrontendFileSync(lambda f, c: self._syncFileHook(f, c))
        self._dirSyncMap = list()

    def _loadPluginConfigs(self) -> [PluginDocDetail]:
        pluginDetails = []

        for plugin in self._loadedPlugins:
            assert isinstance(plugin.packageCfg, PluginPackageFileConfig)
            pluginPackageConfig = plugin.packageCfg.config

            if not self._configKey in pluginPackageConfig.requiresServices([]):
                continue

            if not self._configKey in pluginPackageConfig:
                logger.info("Skipping doc build for %s,"
                            "missing config section for %s",
                    plugin.name, self._platformService)
                continue

            jsonCfgNode = pluginPackageConfig[self._configKey]

            docDir = jsonCfgNode.docDir(None)
            docRst = jsonCfgNode.docRst(None)
            hasApi = jsonCfgNode.hasApi(False)

            pluginDetails.append(
                PluginDocDetail(pluginRootDir=plugin.rootDir,
                    pluginName=plugin.name,
                    pluginTitle=plugin.title,
                    docDir=docDir,
                    docRst=docRst,
                    hasApi=hasApi)
            )

        pluginDetails.sort(key=lambda x: x.pluginName)
        return pluginDetails

    def _writePluginsToc(self, docDir: str,
                         pluginDetails: [PluginDocDetail]) -> None:

        contents = _mainTocTemplate

        for pluginDetail in pluginDetails:

            hasIndex = pluginDetail.docDir and pluginDetail.docRst
            hasApi = pluginDetail.hasApi

            if not (hasIndex or hasApi):
                continue
            contents += "    %s_toc\n" % pluginDetail.pluginName

        self._writeFileIfRequired(docDir, 'plugin_toc.rst', contents)

    def _writePluginToc(self, docDir: str,
                        pluginDetails: [PluginDocDetail]) -> None:

        for pluginDetail in pluginDetails:
            hasIndex = pluginDetail.docDir and pluginDetail.docRst
            hasApi = pluginDetail.hasApi

            if not (hasApi or hasIndex):
                continue

            contents = pluginDetail.pluginTitle + "\n"
            contents += "+" * len(pluginDetail.pluginTitle) + "\n"

            contents += _tocTemplate

            if hasIndex:
                contents += "    %s/%s\n" % (
                    pluginDetail.pluginName,
                    pluginDetail.docRst.replace(".rst", "")
                )

            if hasApi:
                contents += "    %s_api/%s\n" % (
                    pluginDetail.pluginName, pluginDetail.pluginName
                )

            contents += "\n\n"

            self._writeFileIfRequired(docDir,
                '%s_toc.rst' % pluginDetail.pluginName,
                contents)

    def _writePluginsApiConf(self, docDir: str,
                             pluginDetails: [PluginDocDetail]) -> None:

        contents = _confPluginsTemplate

        for pluginDetail in pluginDetails:
            if not pluginDetail.hasApi:
                continue

            contents += _confPluginsTemplatePart % (
                pluginDetail.pluginName, pluginDetail.pluginName
            )

        self._writeFileIfRequired(docDir, 'plugin_api_conf.py', contents)

    def _writePluginsApiList(self, docDir: str,
                             pluginDetails: [PluginDocDetail]) -> None:

        contents = ""

        for pluginDetail in pluginDetails:
            if not pluginDetail.hasApi:
                continue

            contents += pluginDetail.pluginName + "\n"

        self._writeFileIfRequired(docDir, 'plugin_api_list.txt', contents)

    def _syncPluginFiles(self, targetDir: str,
                         pluginDetails: [PluginDocDetail],
                         excludeFilesRegex=()) -> None:

        if not os.path.exists(targetDir):
            os.mkdir(targetDir)  # The parent must exist

        # Make a note of the existing items
        currentItems = set()
        createdItems = set()
        for item in os.listdir(targetDir):
            # We're only dealing with directories
            if not os.path.isdir(os.path.join(targetDir, item)):
                continue
            if item.startswith("peek_plugin_") or item.startswith("peek_core_"):
                currentItems.add(item)

        for pluginDetail in pluginDetails:
            if not pluginDetail.docDir:
                continue

            srcDir = os.path.join(pluginDetail.pluginRootDir,
                pluginDetail.docDir)
            if not os.path.exists(srcDir):
                logger.warning("%s doc dir %s doesn't exist",
                    pluginDetail.pluginName, pluginDetail.docDir)
                continue

            createdItems.add(pluginDetail.pluginName)

            linkPath = os.path.join(targetDir, pluginDetail.pluginName)
            self.fileSync.addSyncMapping(srcDir, linkPath,
                excludeFilesRegex=excludeFilesRegex)

        # Delete the items that we didn't create
        for item in currentItems - createdItems:
            path = os.path.join(targetDir, item)
            if os.path.islink(path):
                os.remove(path)
            elif os.path.isdir(path):
                shutil.rmtree(path)
            else:
                os.remove(path)



import json
import logging
import os
import shutil
from collections import namedtuple
from textwrap import dedent
from typing import List, Callable, Optional, Dict

from jsoncfg.value_mappers import require_bool

from peek_platform.build_common.BuilderABC import BuilderABC
from peek_platform.build_frontend.FrontendFileSync import FrontendFileSync
from peek_platform.file_config.PeekFileConfigFrontendDirMixin import (
    PeekFileConfigFrontendDirMixin,
)
from peek_platform.file_config.PeekFileConfigOsMixin import (
    PeekFileConfigOsMixin,
)
from peek_plugin_base.PluginPackageFileConfig import PluginPackageFileConfig

logger = logging.getLogger(__name__)

# Quiten the file watchdog
logging.getLogger("watchdog.observers.inotify_buffer").setLevel(logging.INFO)

PluginDetail = namedtuple(
    "PluginDetail",
    [
        "pluginRootDir",
        "pluginName",
        "pluginTitle",
        "appDir",
        "appModule",
        "cfgDir",
        "cfgModule",
        "moduleDir",
        "assetDir",
        "rootModules",
        "rootServices",
        "rootComponents",
        "icon",
        "homeLinkText",
        "showHomeLink",
        "showInTitleBar",
        "antIcon",
        "titleSvgIcon",
        "titleBarText",
        "configLinkPath",
    ],
)


class BuildTypeEnum:
    ELECTRON = "ELECTRON"
    WEB_OFFICE = "WEB_OFFICE"
    WEB_FIELD = "WEB_FIELD"
    WEB_ADMIN = "WEB_ADMIN"


class FrontendBuilderABC(BuilderABC):
    """Peek App Frontend Installer Mixin

    This class is used for the client and server.

    This class contains the logic for:
        * Linking in the frontend angular components to the frontend project
        * Compiling the frontend project

    :TODO: Use find/sort to generate a string of the files when this was last run.
        Only run it again if anything has changed.

    """

    _CFG_KEYS = {
        BuildTypeEnum.ELECTRON: ["office-electron", "office"],
        BuildTypeEnum.WEB_OFFICE: ["office-web", "office"],
        BuildTypeEnum.WEB_FIELD: ["field-web", "field"],
        BuildTypeEnum.WEB_ADMIN: ["admin"],
    }

    def __init__(
        self,
        frontendProjectDir: str,
        platformService: str,
        buildType: BuildTypeEnum,
        jsonCfg,
        loadedPlugins: List,
    ):
        assert platformService in (
            "peek-field-app",
            "peek-admin-app",
            "peek-office-app",
        ), (
            "Unexpected service %s" % platformService
        )

        self._platformService = platformService
        self._buildType = buildType
        self._jsonCfg = jsonCfg
        self._frontendProjectDir = frontendProjectDir
        self._loadedPlugins = loadedPlugins

        if not isinstance(self._jsonCfg, PeekFileConfigFrontendDirMixin):
            raise Exception(
                "The file config must inherit the"
                " PeekFileConfigFrontendDirMixin"
            )

        if not isinstance(self._jsonCfg, PeekFileConfigOsMixin):
            raise Exception(
                "The file config must inherit the" " PeekFileConfigOsMixin"
            )

        if not os.path.isdir(frontendProjectDir):
            raise Exception("% doesn't exist" % frontendProjectDir)

        self.fileSync = FrontendFileSync(lambda f, c: self._syncFileHook(f, c))
        self._dirSyncMap = list()
        self._fileWatchdogObserver = None

    def _loadPluginConfigs(self) -> [PluginDetail]:
        pluginDetails = []

        for plugin in self._loadedPlugins:
            assert isinstance(plugin.packageCfg, PluginPackageFileConfig)
            pluginPackageConfig = plugin.packageCfg.config
            jsonCfgNode = None

            for configKey in self._CFG_KEYS[self._buildType]:
                if configKey in pluginPackageConfig:
                    jsonCfgNode = pluginPackageConfig[configKey]
                    break

            if not jsonCfgNode:
                logger.info(
                    "Skipping frontend build for %s,"
                    "missing config section for %s",
                    plugin.name,
                    self._buildType,
                )
                continue

            enabled = jsonCfgNode.enableAngularFrontend(True, require_bool)

            if not enabled:
                continue

            appDir = jsonCfgNode.appDir(None)
            appModule = jsonCfgNode.appModule(None)

            cfgDir = jsonCfgNode.cfgDir(None)
            cfgModule = jsonCfgNode.cfgModule(None)

            moduleDir = jsonCfgNode.moduleDir(None)
            assetDir = jsonCfgNode.assetDir(None)

            showHomeLink = jsonCfgNode.showHomeLink(True)
            homeLinkText = jsonCfgNode.homeLinkText(plugin.title)
            showInTitleBar = jsonCfgNode.showInTitleBar(False)
            antIcon = jsonCfgNode.antIcon(None)
            titleSvgIcon = jsonCfgNode.titleSvgIcon(None)
            titleBarText = jsonCfgNode.titleBarText(None)
            configLinkPath = jsonCfgNode.configLinkPath(None)

            def checkRootComponents(rootComponent):
                rootComponent["selector"] = rootComponent.get("selector")

            def checkThing(name, data):
                sub = (name, plugin.name)
                if data:
                    assert data["file"], "%s.file is missing for %s" % sub
                    assert data["class"], "%s.class is missing for %s" % sub

                if not data.get("persistent"):
                    data["persistent"] = False

                if not data.get("locatedInAppDir"):
                    data["locatedInAppDir"] = False

                # For services
                data["useClassFile"] = data.get("useClassFile")
                data["useClassClass"] = data.get("useClassClass")
                data["useExistingClass"] = data.get("useExistingClass")

            # Root Modules
            rootModules = jsonCfgNode.rootModules([])
            for rootModule in rootModules:
                checkThing("rootModules", rootModule)

            # Root Services
            rootServices = jsonCfgNode.rootServices([])
            for rootService in rootServices:
                checkThing("rootServices", rootService)

            # Root Components
            rootComponents = jsonCfgNode.rootComponents([])
            for rootComponent in rootComponents:
                checkRootComponents(rootComponent)

            icon = jsonCfgNode.icon(None)

            pluginDetails.append(
                PluginDetail(
                    pluginRootDir=plugin.rootDir,
                    pluginName=plugin.name,
                    pluginTitle=plugin.title,
                    appDir=appDir,
                    appModule=appModule,
                    cfgDir=cfgDir,
                    cfgModule=cfgModule,
                    moduleDir=moduleDir,
                    assetDir=assetDir,
                    rootModules=rootModules,
                    rootServices=rootServices,
                    rootComponents=rootComponents,
                    icon=icon,
                    homeLinkText=homeLinkText,
                    showHomeLink=showHomeLink,
                    showInTitleBar=showInTitleBar,
                    antIcon=antIcon,
                    titleSvgIcon=titleSvgIcon,
                    titleBarText=titleBarText,
                    configLinkPath=configLinkPath,
                )
            )

        pluginDetails.sort(key=lambda x: x.pluginName)
        return pluginDetails

    def _makeModuleOrServicePath(self, pluginDetail, modOrService):
        if modOrService["locatedInAppDir"]:
            return "%s" % pluginDetail.pluginName

        return "@peek/%s" % pluginDetail.pluginName

    def _writePluginHomeLinks(
        self, feAppDir: str, pluginDetails: [PluginDetail]
    ) -> None:
        """
        export const homeLinks = [
            {
                name: 'plugin_noop',
                title: "Noop",
                resourcePath: "/peek_plugin_noop",
                pluginIconPath: "/peek_plugin_noop/home_icon.png",
                svgIcon: "/peek_plugin_noop/home_icon.png"
            }
        ];
        """

        links = []
        for pluginDetail in pluginDetails:
            if not (pluginDetail.appModule and pluginDetail.showHomeLink):
                continue

            links.append(
                dict(
                    name=pluginDetail.pluginName,
                    title=pluginDetail.homeLinkText,
                    resourcePath="/%s" % pluginDetail.pluginName,
                    pluginIconPath=pluginDetail.icon,
                    svgIcon=pluginDetail.titleSvgIcon,
                )
            )

        links.sort(key=lambda item: item["title"])

        contents = "// This file is auto generated, the git version is blank and .gitignored\n"
        contents += "export const homeLinks = %s;\n" % json.dumps(
            links, sort_keys=True, indent=4, separators=(", ", ": ")
        )
        self._writeFileIfRequired(feAppDir, "plugin-home-links.ts", contents)

    def _writePluginConfigLinks(
        self, feAppDir: str, pluginDetails: [PluginDetail]
    ) -> None:
        """
        export const configLinks = [
            {
                name: 'plugin_noop',
                title: "Noop",
                resourcePath: "/peek_plugin_noop_cfg",
                pluginIconPath: "/assets/peek_plugin_noop/home_icon.png",
                svgIcon=pluginDetail.titleSvgIcon,
                resourcePath="/%s" % pluginDetail.pluginName
            }
        ];
        """

        links = []
        for pluginDetail in pluginDetails:
            if not pluginDetail.cfgModule:
                continue

            links.append(
                dict(
                    name=pluginDetail.pluginName,
                    title=pluginDetail.homeLinkText,
                    resourcePath="/%s_cfg/" % pluginDetail.pluginName,
                    pluginIconPath=pluginDetail.icon,
                )
            )

        links.sort(key=lambda item: item["title"])

        contents = "// This file is auto generated, the git version is blank and .gitignored\n"
        contents += "export const configLinks = %s;\n" % json.dumps(
            links, sort_keys=True, indent=4, separators=(", ", ": ")
        )

        self._writeFileIfRequired(feAppDir, "plugin-config-links.ts", contents)

    def _writePluginTitleBarLinks(
        self, feAppDir: str, pluginDetails: [PluginDetail]
    ) -> None:
        """

        import {TitleBarLink} from "@synerty/peek-plugin-base-js";

        export const titleBarLinks :TitleBarLink = [
            {
                plugin : "peek_plugin_noop",
                text: "Noop",
                antIcon: "smile",
                resourcePath: "/peek_plugin_noop/home_icon.png",
                badgeCount : null
            }
        ];
        """

        links = []
        for pluginDetail in pluginDetails:
            if not (pluginDetail.appModule and pluginDetail.showInTitleBar):
                continue

            links.append(
                dict(
                    plugin=pluginDetail.pluginName,
                    text=pluginDetail.titleBarText,
                    antIcon=pluginDetail.antIcon,
                    svgIcon=pluginDetail.titleSvgIcon,
                    resourcePath="/%s" % pluginDetail.pluginName,
                    badgeCount=None,
                )
            )

        contents = "// This file is auto generated, the git version is blank and .gitignored\n\n"
        contents += (
            "import { IHeaderLink } from '@synerty/peek-plugin-base-js'\n\n"
        )
        contents += (
            "export const titleBarLinks: IHeaderLink[] = %s;\n"
            % json.dumps(
                links, sort_keys=True, indent=4, separators=(", ", ": ")
            )
        )

        self._writeFileIfRequired(
            feAppDir, "plugin-title-bar-links.ts", contents
        )

    def _writePluginAppRouteLazyLoads(
        self, feAppDir: str, pluginDetails: [PluginDetail]
    ) -> None:
        _appRoutesTemplate = dedent(
            """
            {
                path: '%s',
                loadChildren: "@_peek/%s/%s"
            }"""
        )

        routes = []
        for pluginDetail in pluginDetails:
            if pluginDetail.appModule:
                routes.append(
                    _appRoutesTemplate
                    % (
                        pluginDetail.pluginName,
                        pluginDetail.pluginName,
                        pluginDetail.appModule,
                    )
                )

        routeData = "// This file is auto generated, the git version is blank and .gitignored\n"
        routeData += "export const pluginAppRoutes = ["
        routeData += ",".join(routes)
        routeData += "\n];\n"

        self._writeFileIfRequired(feAppDir, "plugin-app-routes.ts", routeData)

    def _writePluginCfgRouteLazyLoads(
        self, feAppDir: str, pluginDetails: [PluginDetail]
    ) -> None:
        _cfgRoutesTemplate = dedent(
            """
            {
                path: '%s_cfg',
                loadChildren: "@_peek/%s_cfg/%s"
            }"""
        )

        routes = []
        for pluginDetail in pluginDetails:
            if pluginDetail.cfgModule:
                routes.append(
                    _cfgRoutesTemplate
                    % (
                        pluginDetail.pluginName,
                        pluginDetail.pluginName,
                        pluginDetail.cfgModule,
                    )
                )

        routeData = "// This file is auto generated, the git version is blank and .gitignored\n"
        routeData += "export const pluginCfgRoutes = ["
        routeData += ",".join(routes)
        routeData += "\n];\n"

        self._writeFileIfRequired(feAppDir, "plugin-cfg-routes.ts", routeData)

    def _writePluginRootModules(
        self, feAppDir: str, pluginDetails: [PluginDetail]
    ) -> None:

        # initialise the arrays, and put in the persisted service module
        imports = [
            '''import { PluginRootServicePersistentLoadModule } from "./plugin-root-services"'''
        ]
        modules = ["PluginRootServicePersistentLoadModule"]

        for pluginDetail in pluginDetails:
            for rootModule in pluginDetail.rootModules:
                filePath = self._makeModuleOrServicePath(
                    pluginDetail, rootModule
                )

                if filePath.startswith("peek_"):
                    filePath = "@_peek/" + filePath

                imports.append(
                    'import { %s } from "%s/%s"'
                    % (rootModule["class"], filePath, rootModule["file"])
                )
                modules.append(rootModule["class"])

        routeData = "// This file is auto generated, the git version is blank and .gitignored\n"
        routeData += "\n".join(imports) + "\n"
        routeData += "export const pluginRootModules = [\n\t"
        routeData += ",\n\t".join(modules)
        routeData += "\n];\n"

        self._writeFileIfRequired(feAppDir, "plugin-root-modules.ts", routeData)

    def _writePluginRootServices(
        self, feAppDir: str, pluginDetails: [PluginDetail]
    ) -> None:

        imports = []
        services = []
        persistentServices = []
        for pluginDetail in pluginDetails:
            for rootService in pluginDetail.rootServices:
                filePath = self._makeModuleOrServicePath(
                    pluginDetail, rootService
                )

                if filePath.startswith("peek_"):
                    filePath = "@_peek/" + filePath

                imports.append(
                    'import { %s } from "%s/%s"'
                    % (rootService["class"], filePath, rootService["file"])
                )

                if rootService["useClassFile"] and rootService["useClassClass"]:
                    imports.append(
                        'import { %s } from "%s/%s"'
                        % (
                            rootService["useClassClass"],
                            filePath,
                            rootService["useClassFile"],
                        )
                    )
                    services.append(
                        "{provide:%s, useClass:%s}"
                        % (rootService["class"], rootService["useClassClass"])
                    )

                elif rootService["useExistingClass"]:
                    services.append(
                        "{provide:%s, useExisting:%s}"
                        % (
                            rootService["class"],
                            rootService["useExistingClass"],
                        )
                    )

                else:
                    services.append(rootService["class"])

                if rootService["persistent"]:
                    persistentServices.append(rootService["class"])

        routeData = "// This file is auto generated, the git version is blank and .gitignored\n"
        routeData += "\n".join(imports) + "\n"
        routeData += "export const pluginRootServices = [\n\t"
        routeData += ",\n\t".join(services)
        routeData += "\n];\n"

        routeData += """
        import { NgModule } from "@angular/core"

        @NgModule({
        
        })
        export class PluginRootServicePersistentLoadModule {
            constructor(%s) { }
        }
        """ % ", ".join(
            ["private _%s:%s" % (s, s) for s in persistentServices]
        )

        self._writeFileIfRequired(
            feAppDir, "plugin-root-services.ts", routeData
        )

    def _writePluginRootComponents(
        self, feAppDir: str, pluginDetails: [PluginDetail]
    ) -> None:

        # initialise the arrays, and put in the persistent service module
        selectors = []

        for pluginDetail in pluginDetails:
            for comp in pluginDetail.rootComponents:
                selectors.append(
                    "    <%(s)s></%(s)s>" % dict(s=comp["selector"])
                )

        html = "<div>\n%s\n</div>\n" % "\n".join(selectors)
        nsXml = "<StackLayout>\n%s\n</StackLayout>\n" % "\n".join(selectors)

        self._writeFileIfRequired(
            feAppDir, "plugin-root.component.web.html", html
        )

    def _syncPluginFiles(
        self,
        targetDir: str,
        pluginDetails: [PluginDetail],
        attrName: str,
        preSyncCallback: Optional[Callable[[], None]] = None,
        postSyncCallback: Optional[Callable[[], None]] = None,
        keepCompiledFilePatterns: Optional[Dict[str, List[str]]] = None,
        excludeFilesRegex=(),
        isCfgDir=False,
    ) -> None:
        cfgPostfix = "_cfg"

        if not os.path.exists(targetDir):
            os.mkdir(targetDir)  # The parent must exist

        # Make a note of the existing items
        currentItems = set()
        createdItems = set()
        for item in os.listdir(targetDir):
            if not item.startswith("peek_plugin_") or item.startswith(
                "peek_core_"
            ):
                continue

            if isCfgDir:
                if item.endswith(cfgPostfix):
                    currentItems.add(item)
            else:
                if not item.endswith(cfgPostfix):
                    currentItems.add(item)

        for pluginDetail in pluginDetails:
            frontendDir = getattr(pluginDetail, attrName, None)
            if not frontendDir:
                continue

            srcDir = os.path.join(pluginDetail.pluginRootDir, frontendDir)
            if not os.path.exists(srcDir):
                logger.warning(
                    "%s FE dir %s doesn't exist",
                    pluginDetail.pluginName,
                    frontendDir,
                )
                continue

            if isCfgDir:
                createdItems.add(pluginDetail.pluginName + cfgPostfix)
                linkPath = os.path.join(
                    targetDir, pluginDetail.pluginName + cfgPostfix
                )
            else:
                createdItems.add(pluginDetail.pluginName)
                linkPath = os.path.join(targetDir, pluginDetail.pluginName)

            self.fileSync.addSyncMapping(
                srcDir,
                linkPath,
                keepCompiledFilePatterns=keepCompiledFilePatterns,
                preSyncCallback=preSyncCallback,
                postSyncCallback=postSyncCallback,
                excludeFilesRegex=excludeFilesRegex,
            )

        # Delete the items that we didn't create
        for item in currentItems - createdItems:
            path = os.path.join(targetDir, item)
            if os.path.islink(path):
                os.remove(path)
            elif os.path.isdir(path):
                shutil.rmtree(path)
            else:
                os.remove(path)

    def _updatePackageJson(
        self, targetJson: str, pluginDetails: [PluginDetail]
    ) -> None:

        serviceName = "@peek"

        # Remove all the old symlinks

        with open(targetJson, "r") as f:
            jsonData = json.load(f)

        dependencies = jsonData["dependencies"]
        for key in list(dependencies):
            if key.startswith(serviceName):
                del dependencies[key]

        for pluginDetail in pluginDetails:
            if not pluginDetail.moduleDir:
                continue

            moduleDir = os.path.join(
                pluginDetail.pluginRootDir, pluginDetail.moduleDir
            )

            name = "%s/%s" % (serviceName, pluginDetail.pluginName)
            dependencies[name] = "file:///" + moduleDir.replace("\\", "/")

        contents = json.dumps(
            jsonData, sort_keys=True, indent=2, separators=(",", ": ")
        )

        self._writeFileIfRequired(
            os.path.dirname(targetJson), os.path.basename(targetJson), contents
        )



import logging
from datetime import datetime

import os
from typing import List

import pytz

from peek_platform.build_frontend.FrontendBuilderABC import (
    FrontendBuilderABC,
    BuildTypeEnum,
)
from peek_platform.build_common.BuilderOsCmd import runNgBuild
from vortex.DeferUtil import deferToThreadWrapWithLogger

logger = logging.getLogger(__name__)


class WebBuilder(FrontendBuilderABC):
    def __init__(
        self,
        frontendProjectDir: str,
        platformService: str,
        jsonCfg,
        loadedPlugins: List,
    ):
        FrontendBuilderABC.__init__(
            self,
            frontendProjectDir,
            platformService,
            self._buildType(platformService),
            jsonCfg,
            loadedPlugins,
        )

        self.isField = "field" in platformService
        self.isOffice = "office" in platformService
        self.isAdmin = "admin" in platformService

    @staticmethod
    def _buildType(platformService: str):
        if "field" in platformService:
            return BuildTypeEnum.WEB_FIELD
        if "office" in platformService:
            return BuildTypeEnum.WEB_OFFICE
        if "admin" in platformService:
            return BuildTypeEnum.WEB_ADMIN

        raise NotImplementedError("Unknown build type")

    @deferToThreadWrapWithLogger(logger, checkMainThread=False)
    def build(self) -> None:
        if not self._jsonCfg.feWebBuildPrepareEnabled:
            logger.info(
                "%s SKIPPING, Web build prepare is disabled in config",
                self._platformService,
            )
            return

        excludeRegexp = (r".*__pycache__.*", r".*[.]py$")

        if self.isField:
            excludeRegexp += (
                r".*[.]dweb[.]ts$",
                r".*[.]dweb[.]html$",
                r".*[.]dweb[.]scss",
            )

        elif self.isOffice:
            excludeRegexp += (
                r".*[.]mweb[.]ts$",
                r".*[.]mweb[.]html$",
                r".*[.]mweb[.]scss",
            )

        elif self.isAdmin:
            pass

        else:
            raise NotImplementedError(
                "This is neither admin, field or office web"
            )

        self._dirSyncMap = list()

        feBuildDir = self._frontendProjectDir
        feBuildSrcDir = os.path.join(feBuildDir, "src")
        feBuildAssetsDir = os.path.join(feBuildDir, "src", "assets")
        feNodeModDir = os.path.join(feBuildDir, "node_modules")
        fePluginDir = os.path.join(feBuildSrcDir, "@_peek")
        fePrivatePluginDir = os.path.join(feBuildSrcDir, "@peek")
        feModuleDirs = [
            (fePrivatePluginDir, "moduleDir"),
        ]

        if not os.path.exists(fePluginDir):
            os.makedirs(fePluginDir)

        if not os.path.exists(fePrivatePluginDir):
            os.makedirs(fePrivatePluginDir)

        pluginDetails = self._loadPluginConfigs()

        # --------------------
        # Check if node_modules exists

        if not os.path.exists(os.path.join(feBuildDir, "node_modules")):
            raise NotADirectoryError(
                "%s node_modules doesn't exist, ensure you've run "
                "`npm install` in dir %s",
                self._platformService,
                feBuildDir,
            )

        # --------------------
        # Prepare the common frontend application

        # self.fileSync.addSyncMapping(feSrcAppDir, os.path.join(feBuildSrcDir, 'app'),
        #                              excludeFilesRegex=excludeRegexp)

        # --------------------
        # Prepare the home and title bar configuration for the plugins
        self._writePluginHomeLinks(fePluginDir, pluginDetails)
        self._writePluginTitleBarLinks(fePluginDir, pluginDetails)
        self._writePluginConfigLinks(fePluginDir, pluginDetails)

        # --------------------
        # Prepare the plugin lazy loaded part of the application
        self._writePluginAppRouteLazyLoads(fePluginDir, pluginDetails)
        self._syncPluginFiles(
            fePluginDir,
            pluginDetails,
            "appDir",
            excludeFilesRegex=excludeRegexp,
        )

        # --------------------
        # Prepare the plugin lazy loaded part of the application
        self._writePluginCfgRouteLazyLoads(fePluginDir, pluginDetails)
        self._syncPluginFiles(
            fePluginDir,
            pluginDetails,
            "cfgDir",
            isCfgDir=True,
            excludeFilesRegex=excludeRegexp,
        )

        # --------------------
        # Prepare the plugin assets
        self._syncPluginFiles(
            feBuildAssetsDir,
            pluginDetails,
            "assetDir",
            excludeFilesRegex=excludeRegexp,
        )

        # --------------------
        # Prepare the shared / global parts of the plugins

        self._writePluginRootModules(fePluginDir, pluginDetails)
        self._writePluginRootServices(fePluginDir, pluginDetails)
        self._writePluginRootComponents(fePluginDir, pluginDetails)

        for (
            feModDir,
            jsonAttr,
        ) in feModuleDirs:
            # Link the shared code, this allows plugins
            # * to import code from each other.
            # * provide global services.
            self._syncPluginFiles(
                feModDir,
                pluginDetails,
                jsonAttr,
                excludeFilesRegex=excludeRegexp,
            )

        # Lastly, Allow the clients to override any frontend files they wish.
        # Src Directory
        self.fileSync.addSyncMapping(
            self._jsonCfg.feFrontendSrcOverlayDir,
            fePluginDir,
            parentMustExist=True,
            deleteExtraDstFiles=False,
            excludeFilesRegex=excludeRegexp,
        )

        # node_modules Directory
        self.fileSync.addSyncMapping(
            self._jsonCfg.feFrontendNodeModuleOverlayDir,
            feNodeModDir,
            parentMustExist=True,
            deleteExtraDstFiles=False,
            excludeFilesRegex=excludeRegexp,
        )

        self.fileSync.syncFiles()

        if self._jsonCfg.feSyncFilesForDebugEnabled:
            logger.info(
                "%s starting frontend development file sync",
                self._platformService,
            )
            self.fileSync.startFileSyncWatcher()

        if self._jsonCfg.feWebBuildEnabled:
            logger.info("%s starting frontend web build", self._platformService)
            self._compileFrontend(feBuildDir)

    def _syncFileHook(self, fileName: str, contents: bytes) -> bytes:
        # replace imports that end with .dweb or .mweb to the appropriate
        # value
        # Otherwise just .web should be used if no replacing is required.

        for lineEnd in (os.linesep.encode(), (";" + os.linesep).encode()):

            if self.isField:
                contents = contents.replace(
                    b'.dweb"' + lineEnd, b'.mweb"' + lineEnd
                )

            elif self.isOffice:
                contents = contents.replace(
                    b'.mweb"' + lineEnd, b'.dweb"' + lineEnd
                )

            elif self.isAdmin:
                pass

            else:
                raise NotImplementedError("This is neither field or office web")

        if b"@Component" in contents:
            return self._patchComponent(fileName, contents)

        return contents

    def _patchComponent(self, fileName: str, contents: bytes) -> bytes:
        """Patch Component

        Apply patches to the WEB file to convert it to the NativeScript version

        :param fileName: The name of the file
        :param contents: The original contents of the file
        :return: The new contents of the file
        """
        inComponentHeader = False

        newContents = b""
        for line in contents.splitlines(True):
            if line.startswith(b"@Component"):
                inComponentHeader = True

            elif line.startswith(b"export"):
                inComponentHeader = False

            elif inComponentHeader:

                if self.isOffice:
                    line = (
                        line.replace(b".mweb.html", b".dweb.html")
                        .replace(b".mweb.css", b".dweb.css")
                        .replace(b".mweb.scss", b".dweb.scss")
                    )

                if self.isField:
                    line = (
                        line.replace(b".dweb.html", b".mweb.html")
                        .replace(b".dweb.css", b".mweb.css")
                        .replace(b".dweb.scss", b".mweb.scss")
                    )

            newContents += line

        return newContents

    def _compileFrontend(self, feBuildDir: str) -> None:
        """Compile the frontend

        this runs `ng build`

        We need to use a pty otherwise webpack doesn't run.

        """
        startDate = datetime.now(pytz.utc)
        hashFileName = os.path.join(feBuildDir, ".lastHash")

        if not self._recompileRequiredCheck(feBuildDir, hashFileName):
            logger.info(
                "%s Frontend has not changed, recompile not required.",
                self._platformService,
            )
            return

        logger.info(
            "%s Rebuilding frontend distribution", self._platformService
        )

        try:
            runNgBuild(feBuildDir)

        except Exception as e:
            if os.path.exists(hashFileName):
                os.remove(hashFileName)

            # Update the detail of the exception and raise it
            e.message = (
                "%s angular frontend failed to build." % self._platformService
            )
            raise

        logger.info(
            "%s frontend rebuild completed in %s",
            self._platformService,
            datetime.now(pytz.utc) - startDate,
        )



import logging
import os
from abc import ABCMeta
from typing import Optional

from jsoncfg.value_mappers import (
    require_string,
    RequireType,
    require_list,
    require_bool,
    require_integer,
)

logger = logging.getLogger(__name__)


class PeekFileConfigPlatformMixin(metaclass=ABCMeta):
    # --- Platform Logging

    @property
    def loggingDebugMemoryMask(self) -> int:
        with self._cfg as c:
            return c.logging.debugMemoryMask(0, require_integer)

    @property
    def loggingLevel(self) -> str:
        with self._cfg as c:
            lvl = c.logging.level("INFO", require_string)
            if lvl in logging._nameToLevel:
                return lvl

            logger.warning("Logging level %s is not valid, defauling to INFO", lvl)
            return "INFO"

    @property
    def logToStdout(self) -> str:
        with self._cfg as c:
            return c.logging.logToStdout(False, require_bool)

    @property
    def loggingRotateSizeMb(self) -> int:
        with self._cfg as c:
            return c.logging.rotateSizeMb(20, require_integer)

    @property
    def loggingRotationsToKeep(self) -> int:
        with self._cfg as c:
            return c.logging.rotationsToKeep(2, require_integer)

    @property
    def loggingLogToSyslogHost(self) -> Optional[str]:
        with self._cfg as c:
            return c.logging.syslog.logToSysloyHost(None)

    @property
    def loggingLogToSyslogPort(self) -> int:
        with self._cfg as c:
            return c.logging.syslog.logToSysloyPort(514, require_integer)

    @property
    def loggingLogToSyslogFacility(self) -> str:
        with self._cfg as c:
            return c.logging.syslog.logToSysloyProtocol("user", require_string)

    @property
    def twistedThreadPoolSize(self) -> int:
        with self._cfg as c:
            count = c.twisted.threadPoolSize(500, require_integer)

        # Ensure the thread count is high
        if count < 50:
            logger.info("Upgrading thread count from %s to %s", count, 500)
            count = 500
            with self._cfg as c:
                c.twisted.threadPoolSize = count

        return count

    @property
    def autoPackageUpdate(self):
        with self._cfg as c:
            return c.platform.autoPackageUpdate(True, require_bool)

    # --- Platform Tmp Path
    @property
    def tmpPath(self):
        default = os.path.join(self._homePath, "tmp")
        with self._cfg as c:
            return self._chkDir(c.disk.tmp(default, require_string))

    # --- Platform Software Path
    @property
    def platformSoftwarePath(self):
        default = os.path.join(self._homePath, "platform_software")
        with self._cfg as c:
            return self._chkDir(c.platform.softwarePath(default, require_string))

    # --- Platform Version
    @property
    def platformVersion(self):
        with self._cfg as c:
            return c.platform.version("0.0.0", require_string)

    @platformVersion.setter
    def platformVersion(self, value):
        with self._cfg as c:
            c.platform.version = value

    # --- Plugin Software Path
    @property
    def pluginSoftwarePath(self):
        default = os.path.join(self._homePath, "plugin_software")
        with self._cfg as c:
            return self._chkDir(c.plugin.softwarePath(default, require_string))

    # --- Plugin Data Path
    def pluginDataPath(self, pluginName):
        default = os.path.join(self._homePath, "plugin_data")

        with self._cfg as c:
            pluginData = c.plugin.dataPath(default, require_string)

        return self._chkDir(os.path.join(pluginData, pluginName))

    # --- Plugin Software Version
    def pluginVersion(self, pluginName):
        """Plugin Version

        The last version that we know about
        """
        with self._cfg as c:
            return c.plugin[pluginName].version(None, RequireType(type(None), str))

    def setPluginVersion(self, pluginName, version):
        with self._cfg as c:
            c.plugin[pluginName].version = version

    # --- Plugins Installed
    @property
    def pluginsEnabled(self):
        with self._cfg as c:
            return c.plugin.enabled([], require_list)

    @pluginsEnabled.setter
    def pluginsEnabled(self, value):
        with self._cfg as c:
            c.plugin.enabled = value



import logging
import subprocess
from typing import List

from peek_platform.WindowsPatch import isWindows
from peek_platform.util.PtyUtil import PtyOutParser, spawnPty, logSpawnException

logger = logging.getLogger(__name__)

NG_BUILD_ARGS = "ng build --prod --optimization  --common-chunk --vendor-chunk".split()


def runDocBuild(feBuildDir: str):
    if isWindows:
        return __runNodeCmdWin(feBuildDir, ["bash", "./build_html_docs.sh"])
    return __runNodeCmdLin(feBuildDir, ["bash", "./build_html_docs.sh"])


def runNgBuild(feBuildDir: str):
    if isWindows:
        return __runNodeCmdWin(feBuildDir, NG_BUILD_ARGS)
    return __runNodeCmdLin(feBuildDir, NG_BUILD_ARGS)


def runTsc(feDir: str):
    if isWindows:
        return __runNodeCmdWin(feDir, ["tsc"])
    return __runNodeCmdLin(feDir, ["tsc"])


def __runNodeCmdWin(feBuildDir: str, cmds: List[str]):
    proc = subprocess.Popen(
        cmds, cwd=feBuildDir, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True
    )

    logger.info("Wating up to 5m for command to finish")
    outs, errs = proc.communicate(timeout=300)

    if proc.returncode in (0,):
        logger.info("%s finished successfully" % " ".join(cmds))
        # for line in (outs + errs).decode().splitlines():
        #     print(".")
    else:
        for line in (outs + errs).decode().splitlines():
            print(line)

        raise Exception("%s in %s failed" % (" ".join(cmds), feBuildDir))

    logger.info("Command complete")


def __runNodeCmdLin(feBuildDir: str, cmds: List[str]):
    try:
        parser = PtyOutParser(loggingStartMarker="Hash: ")
        spawnPty("cd %s && %s" % (feBuildDir, " ".join(cmds)), parser)

    except Exception as e:
        logSpawnException(e)
        raise



import logging
import os
import shutil
import unittest

import peek_platform
from jsoncfg.functions import config_to_json_str
from peek_platform.file_config.PeekFileConfigABC import PeekFileConfigABC
from peek_platform.file_config.PeekFileConfigPeekServerClientMixin import (
    PeekFileConfigPeekServerClientMixin,
)
from peek_platform.file_config.PeekFileConfigPlatformMixin import (
    PeekFileConfigPlatformMixin,
)

logging.basicConfig(level=logging.DEBUG)

logger = logging.getLogger(__name__)


class TestFileConfig(
    PeekFileConfigABC, PeekFileConfigPeekServerClientMixin, PeekFileConfigPlatformMixin
):
    pass


class PeekFileConfigTest(unittest.TestCase):
    COMPONENT_NAME = "unit_test"
    HOME_DIR = os.path.expanduser("~/%s.home" % COMPONENT_NAME)
    CONFIG_FILE_PATH = os.path.join(HOME_DIR, "config.json")

    def _rmHome(self):
        if os.path.exists(self.HOME_DIR):
            shutil.rmtree(self.HOME_DIR)

    def setUp(self):
        TestFileConfig._PeekFileConfigBase__instance = None
        peek_platform.PeekPlatformConfig.componentName = "unit_test"
        self._rmHome()
        os.makedirs(self.HOME_DIR, TestFileConfig.DEFAULT_DIR_CHMOD)

        with open(self.CONFIG_FILE_PATH, "w") as fobj:
            fobj.write('{"nothing":{"is_true":true}}')

    def tearDown(self):
        self._rmHome()

    def testReadingConfig(self):
        bas = TestFileConfig()

        logger.info("nothing.is_true = %s", bas._cfg.nothing.is_true)
        self.assertTrue(bas._cfg.nothing.is_true())

    def testPeekServerDetails(self):
        bas = TestFileConfig()
        logger.info("peekServerPort = %s", bas.peekServerPort)
        logger.info("peekServerHost = %s", bas.peekServerHost)

        logger.info(config_to_json_str(bas._cfg))

        bas._save()

        with open(self.CONFIG_FILE_PATH, "r") as fobj:
            logger.info(fobj.read())

    def testAsignment(self):
        inArr = [1, 2, 5, "20"]
        valuey = "valuey"
        twoFiveSix = 256

        bas = TestFileConfig()

        bas._cfg.op1.thingx = valuey
        bas._cfg.op1.thingy = twoFiveSix
        bas._cfg.op2.op3.arr = inArr

        logger.info(config_to_json_str(bas._cfg))

        bas._save()

        with open(self.CONFIG_FILE_PATH, "r") as fobj:
            logger.info(fobj.read())

        TestFileConfig._PeekFileConfigBase__instance = None

        bas = TestFileConfig()
        self.assertEqual(bas._cfg.op2.op3.arr(), inArr)
        self.assertEqual(bas._cfg.op1.thingx(), valuey)
        self.assertEqual(bas._cfg.op1.thingy(), twoFiveSix)

    def testPlatformDetails(self):
        pluginName = "plugin_noop"

        bas = TestFileConfig()

        # Defaults
        logger.info("plugin.plugin_noop.version = %s", bas.pluginVersion(pluginName))
        logger.info("platformVersion = %s", bas.platformVersion)

        bas.platformVersion = "4.4.4"
        bas.setPluginVersion(pluginName, "2.5.6")

        with open(self.CONFIG_FILE_PATH, "r") as fobj:
            logger.info(fobj.read())

        TestFileConfig._PeekFileConfigBase__instance = None

        logger.info("plugin.plugin_noop.version = %s", bas.pluginVersion(pluginName))
        logger.info("platformVersion = %s", bas.platformVersion)

        self.assertEqual(bas.platformVersion, "4.4.4")
        self.assertEqual(bas.pluginVersion(pluginName), "2.5.6")



import logging
import os
from typing import Optional

from jsoncfg.value_mappers import require_string, require_bool, require_integer
from peek_platform.file_config.PeekFileConfigPlatformMixin import (
    PeekFileConfigPlatformMixin,
)

logger = logging.getLogger(__name__)


class PeekFileConfigHttpMixin:
    def __init__(
        self, config: PeekFileConfigPlatformMixin, name: str, defaultPort: int
    ):
        self._config = config
        self._name = name
        self._defaultPort = defaultPort

    ### SERVER SECTION ###
    @property
    def sitePort(self) -> int:
        with self._config._cfg as c:
            return c.httpServer[self._name].sitePort(self._defaultPort, require_integer)

    @property
    def enableHttps(self) -> int:
        with self._config._cfg as c:
            return c.httpServer[self._name].enableHttps(False, require_bool)

    @property
    def redirectFromHttpPort(self) -> int:
        with self._config._cfg as c:
            return c.httpServer[self._name].redirectFromHttpPort(None)

    @property
    def sslBundleFilePath(self) -> Optional[str]:
        default = os.path.join(self._config._homePath, "peek-ssl-bundle.pem")
        with self._config._cfg as c:
            file = c.httpServer[self._name].sslBundleFilePath(default, require_string)
            if os.path.exists(file):
                return file
            return None
